/*
 * File:   Main.c
 * Author: Admin
 *
 * Created on 22. Mrz 2024, 18:03
 */


#include <xc.h>
#include "mcc.h"
#include <stdint.h>

uint8_t  i, second, minute, hour, w_day, m_day, month, year,
         alarm1_minute, alarm1_hour, alarm2_minute, alarm2_hour,
         status_reg, alarm1_status, alarm2_status;

/*********************************** I2C functions ****************************/

void I2C_Init()
{
    SSP1CON1  = 0x28;   // configure MSSP module to work in I2C mode
    SSP1ADD  = 0x4F;    // set I2C clock frequency 100khz > SCL pin clock period = ((ADD <7:0> + 1) *4)/FOSC
    SSP1STAT = 0;       // (_XTAL_FREQ /(4 * i2c_clk_freq))-1 = (32000000hz/(4*100000hz) -1 = 79 = 0x4F
}
   
void I2C_Start()
{
    while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
    SEN1 = 1;  // initiate start condition
}

void I2C_Repeated_Start()
{
  while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
  RSEN1 = 1;  // initiate repeated start condition
}

void I2C_Stop()
{
  while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
  PEN1 = 1;  // initiate stop condition
}

void I2C_Write(uint8_t i2c_data)
{
  while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
  SSP1BUF = i2c_data;  // update buffer
}

uint8_t I2C_Read(uint8_t ack)
{
  uint8_t _data;
  while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
  RCEN1 = 1;
  while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
  _data = SSP1BUF;  // read data from buffer
  while ((SSP1STAT & 0x04) || (SSP1CON2 & 0x1F));  // wait for MSSP module to be free (not busy)
  // send acknowledge pulse ? (depends on ack, if 1 send, otherwise don't send)
  ACKDT1 = !ack;
  ACKEN1 = 1;
  return _data;  // return data read
}
/*********************** end I2C functions ************************************/
void RTC_read()    // read current time & date from RTC chip
{
  I2C_Start();           // start I2C
  I2C_Write(0xD0);       // RTC chip address
  I2C_Write(0x00);          // send register address
  I2C_Repeated_Start();  // restart I2C
  I2C_Write(0xD1);       // initialize data read
  second = I2C_Read(1);  // read seconds from register 0
  minute = I2C_Read(1);  // read minutes from register 1
  hour   = I2C_Read(1);  // read hour from register 2
  w_day  = I2C_Read(1);  // read day of the week from register 3
  m_day  = I2C_Read(1);  // read date from register 4
  month  = I2C_Read(1);  // read month from register 5
  year   = I2C_Read(0);  // read year from register 6
  I2C_Stop();            // stop I2C
}


/*************************** main function ************************************/
void main(void)
{
  SYSTEM_Initialize();    // initialize the device 
  OSCFRQ = 0x03;   // set internal oscillator to 8MHz
  ANSELB = 0;      // configure all PORTB pins as digital
  PORTB  = 0;      // PORTB initial state
  TRISB  = 0x0F;   // configure RB0 ~ 3 as input pins
  // enable RB0, RB1, RB2 and RB3 internal pull ups
  //  = 0;      // clear RBPU bit (OPTION_REG.7)
  WPUB   = 0x0F;   // WPUB register = 0b00001111

  INTE   = 0;      // clear external interrupt flag bit
  GIE    = 1;      // enable global interrupts
  INTEDG = 0;      // set external Interrupt on falling edge
  INTE   = 1;      // enable external interrupt

  __delay_ms(1000);  // wait 1 second

 I2C_Init();   // initialize I2C bus with clock frequency of 100kHz
 while(1) {
 RTC_read();             // read time and date from RTC chip
 }
}
/*************************** end main function ********************************/
/**
 End of File
*/